<?php

namespace iwetcl\classes\controllers;

use iwetcl\classes\helpers\Builder_Helper;
use iwetcl\classes\helpers\Sanitizer;
use iwetcl\classes\repositories\builder\Email_Type;
use iwetcl\classes\templates\Template;
use iwetcl\classes\repositories\Order;
use iwetcl\classes\repositories\Post;
use iwetcl\classes\repositories\Product;

class Email_Builder
{
    private static $instance;

    private $post_type;
    private $nonce;
    private $templates;
    private $email_views_dir;
    private $email_type_repository;
    private $grouped_email_types;
    private $post_data;

    public static function init()
    {
        if (is_null(self::$instance)) {
            self::$instance = new self();
        }
    }

    private function __construct()
    {
        $this->nonce = wp_create_nonce();
        $this->email_views_dir = IWETCL_VIEWS_DIR . 'builder/';
        $this->post_type = 'iwetc_emails';

        $this->email_type_repository = Email_Type::get_instance();
        $this->grouped_email_types = $this->email_type_repository->get_grouped_email_types();
        $this->post_data = $this->get_post_data();

        $template_repository = Template::get_instance();
        $this->templates = $template_repository->get_templates();

        add_action('init', [$this, 'register_post_type']);
        add_filter('enter_title_here', [$this, 'set_email_title_text']);
        add_action('admin_head', [$this, 'remove_action'], PHP_INT_MAX);
        add_action('in_admin_header', [$this, 'remove_action'], PHP_INT_MAX);

        add_action('add_meta_boxes', [$this, 'register_meta_box']);
        add_action("save_post_{$this->post_type}", [$this, 'save_post']);
        add_action('admin_enqueue_scripts', [$this, 'load_assets']);

        add_filter("manage_{$this->post_type}_posts_columns", [$this, 'set_column_header']);
        add_action("manage_{$this->post_type}_posts_custom_column", [$this, 'set_column_content'], 10, 2);

        add_filter('post_row_actions', [$this, 'post_row_actions'], 10, 2);
        add_action('post_action_iwetc_email_duplicate', [$this, 'duplicate_template']);

        add_action('woocommerce_email_setting_column_template', array($this, 'add_edit_button'));
        add_filter('parse_query', [$this, 'parse_query_filter']);

        add_filter('admin_body_class', [$this, 'set_theme']);

        $this->customize_safe_style();
    }

    public function post_row_actions($action, $post)
    {
        if ($post->post_type === $this->post_type) {
            unset($action['inline hide-if-no-js']);
            $href = admin_url("post.php?action=iwetc_email_duplicate&id={$post->ID}");
            $action['iwetc-duplicate'] = "<a href='{$href}' onclick='this.style.visibility=\"hidden\";'>" . __('Duplicate', 'ithemeland-woocommerce-email-template-customizer-lite') . "</a>";
        }

        return $action;
    }

    public function set_theme($classes)
    {
        $class_name = get_option('iwetc_theme', 'iwetc-dark-theme');

        if (is_array($classes)) {
            $classes[] = $class_name;
        } else {
            $classes .= ' ' . $class_name;
        }

        return $classes;
    }

    private function customize_safe_style()
    {
        add_filter('safe_style_css', function ($styles) {
            $styles[] = 'display';
            $styles[] = 'flex-wrap';
            $styles[] = 'box-sizing';
            $styles[] = '-moz-box-sizing';
            $styles[] = '-webkit-box-sizing';
            $styles[] = 'flex';
            return $styles;
        });
    }

    public function set_column_header($columns)
    {
        $columns = [
            'cb' => '<input type="checkbox">',
            'title' => __('Email subject', 'ithemeland-woocommerce-email-template-customizer-lite'),
            'type' => __('Type', 'ithemeland-woocommerce-email-template-customizer-lite'),
            'recipient' => __('Recipient', 'ithemeland-woocommerce-email-template-customizer-lite'),
            'template_status' => __('Template status', 'ithemeland-woocommerce-email-template-customizer-lite'),
            'date' => __('Date', 'ithemeland-woocommerce-email-template-customizer-lite')
        ];

        return $columns;
    }

    public function set_column_content($column, $post_id)
    {
        $email_type = get_post_meta($post_id, 'iwetc_email_type', true);

        if ($column == 'type') {
            $type = $this->email_type_repository->get_type(esc_html($email_type));
            echo esc_html($type);
        }

        if ($column == 'recipient') {
            $recipient = $this->email_type_repository->get_email_recipient();
            echo (!empty($recipient[$email_type])) ? __('Admin', 'ithemeland-woocommerce-email-template-customizer-lite')  . ' (' . esc_html($recipient[$email_type]) . ')' : __('Customer', 'ithemeland-woocommerce-email-template-customizer-lite');
        }

        if ($column == 'template_status') {
            $is_enable = get_post_meta($post_id, 'iwetc_template_enable', true);
            $style = "width: 15px; height: 15px; display: inline-table; border-radius: 15px; -moz-border-radius: 15px; -webkit-border-radius: 15px;";
            echo (empty($is_enable) || $is_enable == 'true') ? '<span style="' . $style . ' background: green;"></span>' : '<span style="' . $style . ' background: red;"></span>';
        }
    }

    public function register_post_type()
    {
        $labels = array(
            'name' => _x('Emails', 'Post Type General Name', 'ithemeland-woocommerce-email-template-customizer-lite'),
            'singular_name' => _x('Email', 'Post Type Singular Name', 'ithemeland-woocommerce-email-template-customizer-lite'),
            'menu_name' => sprintf('%s', 'iT Email Builder'),
            'parent_item_colon' => __('Parent Email', 'ithemeland-woocommerce-email-template-customizer-lite'),
            'all_items' => __('All Emails', 'ithemeland-woocommerce-email-template-customizer-lite'),
            'add_new_item' => __('Add New Email', 'ithemeland-woocommerce-email-template-customizer-lite'),
            'add_new' => __('Add New', 'ithemeland-woocommerce-email-template-customizer-lite'),
            'edit_item' => __('Edit Email', 'ithemeland-woocommerce-email-template-customizer-lite'),
            'update_item' => __('Update Email', 'ithemeland-woocommerce-email-template-customizer-lite'),
            'search_items' => __('Search Email', 'ithemeland-woocommerce-email-template-customizer-lite'),
            'not_found' => __('Not Found', 'ithemeland-woocommerce-email-template-customizer-lite'),
            'not_found_in_trash' => __('Not found in Trash', 'ithemeland-woocommerce-email-template-customizer-lite'),
        );

        $args = array(
            'label' => __('Emails', 'ithemeland-woocommerce-email-template-customizer-lite'),
            'labels' => $labels,
            'supports' => ['title'],
            'hierarchical' => false,
            'public' => true,
            'show_ui' => true,
            'menu_icon' => 'dashicons-email-alt',
            'show_in_menu' => false,
            'show_in_nav_menus' => true,
            'show_in_admin_bar' => true,
            'can_export' => true,
            'has_archive' => true,
            'exclude_from_search' => false,
            'publicly_queryable' => false,
            'capability_type' => 'post',
            'query_var' => true,
            'menu_position' => 2,
            'rewrite' => [
                'slug' => $this->post_type
            ],
        );

        register_post_type($this->post_type, $args);
    }

    public function remove_action()
    {
        if (get_current_screen()->id == $this->post_type) {
            remove_all_actions('admin_notices');

            global $wp_meta_boxes;
            if (isset($wp_meta_boxes[$this->post_type]['side'])) {
                unset($wp_meta_boxes[$this->post_type]['side']);
            }
        }
    }

    public function register_meta_box()
    {
        add_meta_box(
            'iwetc-email-builder',
            __('Email', 'ithemeland-woocommerce-email-template-customizer-lite'),
            [$this, 'email_builder_box'],
            $this->post_type,
            'normal',
            'default'
        );
    }

    public function save_post($post_id)
    {
        $data = [
            'iwetc_preload' => (isset($_POST['iwetc_preload'])) ? array_map('sanitize_text_field', $_POST['iwetc_preload']) : '',
            'iwetc_email_type' => (isset($_POST['iwetc_email_type'])) ? sanitize_text_field($_POST['iwetc_email_type']) : '',
            'iwetc_custom_css' => (isset($_POST['iwetc_custom_css'])) ? sanitize_text_field($_POST['iwetc_custom_css']) : '',
            'iwetc_direction' => (isset($_POST['iwetc_direction'])) ? sanitize_text_field($_POST['iwetc_direction']) : 'ltr',
            'iwetc_template_enable' => (isset($_POST['iwetc_template_enable'])) ? sanitize_text_field($_POST['iwetc_template_enable']) : '',
            'iwetc_email_structure' => (!empty($_POST['iwetc_email_structure'])) ? Sanitizer::html($_POST['iwetc_email_structure']) : '',
        ];

        foreach ($data as $key => $value) {
            update_post_meta($post_id, $key, $value);
        }
    }

    public function duplicate_template()
    {
        $email_id = (!empty($_GET['id'])) ? intval($_GET['id']) : '';
        if (!empty($email_id)) {
            $current_email = get_post($email_id);

            $args = [
                'post_title' => $current_email->post_title . ' - copy',
                'post_type' => $current_email->post_type,
            ];
            $new_email_id = wp_insert_post($args);

            $preload = get_post_meta($email_id, 'iwetc_preload', true);
            $email_type = get_post_meta($email_id, 'iwetc_email_type', true);
            $custom_css = get_post_meta($email_id, 'iwetc_custom_css', true);
            $direction = get_post_meta($email_id, 'iwetc_direction', true);
            $is_enable = get_post_meta($email_id, 'iwetc_template_enable', true);
            $structure = get_post_meta($email_id, 'iwetc_email_structure', true);

            update_post_meta($new_email_id, 'iwetc_preload', $preload);
            update_post_meta($new_email_id, 'iwetc_email_type', $email_type);
            update_post_meta($new_email_id, 'iwetc_custom_css', $custom_css);
            update_post_meta($new_email_id, 'iwetc_direction', $direction);
            update_post_meta($new_email_id, 'iwetc_template_enable', $is_enable);
            update_post_meta($new_email_id, 'iwetc_email_structure', str_replace('\\', '\\\\', $structure));

            wp_safe_redirect(admin_url("post.php?post={$new_email_id}&action=edit"));
            exit;
        }
    }

    public function add_edit_button($email)
    {
        $email_types = $this->email_type_repository->get_email_types();
        if (!empty($email_types[$email->id])) {
            $email_type = $email->id;
            $email_title = $email->title;
        } else {
            $email_type = 'default';
            $email_title = 'Default';
        }

        echo '<td>';
        echo sprintf("<a href='%1s' class='button'>%2s</a>", esc_url(admin_url("post-new.php?post_type=iwetc_emails&email_type={$email_type}&email_title=" . urlencode($email_title))), __('Customize with iT Email Builder', 'ithemeland-woocommerce-email-template-customizer-lite'));
        echo '</td>';
    }

    public function parse_query_filter($query)
    {
        global $pagenow;

        if (is_admin() && $pagenow == 'edit.php' && !empty($_GET['post_type']) && $_GET['post_type'] == 'iwetc_emails' && !empty($_GET['email_type'])) {
            $query->query_vars['meta_key'] = 'iwetc_email_type';
            $query->query_vars['meta_value'] = sanitize_text_field($_GET['email_type']);
            $query->query_vars['meta_compare'] = '=';
        }

        return $query;
    }

    public function email_builder_box($post)
    {
        $current_user = wp_get_current_user();
        $order_repository = Order::get_instance();
        $product_repository = Product::get_instance();

        $preload = get_post_meta($post->ID, 'iwetc_preload', true);
        $is_enable = get_post_meta($post->ID, 'iwetc_template_enable', true);

        $email_structure = [];
        if (!empty($this->post_data['email_structure'])) {
            $email_structure = (is_array($this->post_data['email_structure'])) ? $this->post_data['email_structure'] : json_decode($this->post_data['email_structure'], true);
        }
        $trash_nonce = (!empty($post->ID)) ? wp_create_nonce('trash-post_' . $post->ID) : '';

        // if new post
        if (!isset($_GET['action']) && isset($_GET['email_type']) && isset($_GET['email_title'])) {
            echo '<script>
                var iwetcNewEmailData = {
                    emailType: "' . esc_html($_GET['email_type']) . '",
                    emailTitle: "' . esc_html(urldecode($_GET['email_title'])) . '"
                };
            </script>';
        }

        wc_get_template('main.php', [
            'iwetc_nonce' => $this->nonce,
            'trash_nonce' => $trash_nonce,
            'grouped_email_types' => $this->grouped_email_types,
            'post_type' => $this->post_type,
            'post' => $post,
            'is_enable' => $is_enable,
            'current_user_email' => ($current_user instanceof \WP_User) ? $current_user->user_email : '',
            'billing_countries' => $order_repository->get_billing_countries(),
            'product_categories' => $product_repository->get_product_categories(),
            'preload' => $preload,
            'email_structure' => $email_structure,
            'orders' => $order_repository->get_orders_per_status(),
            'saved_templates' => $this->templates,
        ], IWETCL_NAME, $this->email_views_dir);
    }

    public function set_email_title_text($title)
    {
        if (get_current_screen()->id == $this->post_type) {
            $title = __('Add Subject', 'ithemeland-woocommerce-email-template-customizer-lite');
        }

        return $title;
    }

    public function load_assets()
    {
        if (get_current_screen()->id == $this->post_type) {
            // styles
            wp_enqueue_style('iwetc-reset', IWETCL_CSS_URL . 'common/reset.css', [], IWETCL_VERSION);
            wp_enqueue_style('iwetc-select2', IWETCL_CSS_URL . 'common/select2.min.css', [], IWETCL_VERSION);
            wp_enqueue_style('iwetc-sweetalert', IWETCL_CSS_URL . 'common/sweetalert.css', [], IWETCL_VERSION);
            wp_enqueue_style('iwetc-modal', IWETCL_CSS_URL . 'common/modal.css', [], IWETCL_VERSION);
            wp_enqueue_style('iwetc-main', IWETCL_CSS_URL . 'common/style.css', [], IWETCL_VERSION);
            wp_enqueue_style('iwetc-it-builder-font-icon', IWETCL_CSS_URL . 'builder/it-builder-font-icon.css', [], IWETCL_VERSION);
            wp_enqueue_style('iwetc-builder', IWETCL_CSS_URL . 'builder/builder.css', [], IWETCL_VERSION);

            // preload items
            if (!empty($_GET['post'])) {
                $product_repository = Product::get_instance();
                $post_repository = Post::get_instance();

                $products = $product_repository->get_preload_products(intval($_GET['post']));
                $product_categories = $product_repository->get_preload_product_categories(intval($_GET['post']));
                $post_categories = $post_repository->get_preload_post_categories(intval($_GET['post']));
                $post_included = $post_repository->get_preload_post_included(intval($_GET['post']));
                $post_excluded = $post_repository->get_preload_post_excluded(intval($_GET['post']));
            }
            // scripts
            wp_enqueue_editor();
            wp_enqueue_media();
            wp_enqueue_style('wp-color-picker');
            wp_enqueue_script('jquery-ui-sortable');
            wp_enqueue_script('jquery-ui-draggable');
            wp_enqueue_script('iwetc-select2', IWETCL_JS_URL . 'common/select2.min.js', [], IWETCL_VERSION);
            wp_enqueue_script('iwetc-sweetalert', IWETCL_JS_URL . 'common/sweetalert.min.js', [], IWETCL_VERSION);
            wp_enqueue_script('iwetc-constants', IWETCL_JS_URL . 'builder/constants.js', [], IWETCL_VERSION);
            wp_enqueue_script('iwetc-builder', IWETCL_JS_URL . 'builder/builder.js', ['jquery', 'iwetc-constants', 'jquery-ui-draggable', 'jquery-ui-sortable'], IWETCL_VERSION);
            wp_localize_script('iwetc-builder', 'IwetcBuilder', [
                'linkIcon' => '<i class="dashicons dashicons-admin-links"></i>',
                'unlinkIcon' => '<i class="dashicons dashicons-editor-unlink"></i>',
                'Translate' => $this->get_builder_translations(),
                'PlaceholderImageSrc' => IWETCL_IMAGES_URL . 'builder/placeholder-image.jpg',
                'ProductImageSrc' => IWETCL_IMAGES_URL . 'builder/product-image.png',
                'PostImageSrc' => IWETCL_IMAGES_URL . 'builder/post-image.png',
                'SiteIconSrc' => [
                    'black' => IWETCL_IMAGES_URL . 'builder/site-icon-black.png',
                    'white' => IWETCL_IMAGES_URL . 'builder/site-icon-white.png'
                ],
                'EmailIconSrc' => [
                    'black' => IWETCL_IMAGES_URL . 'builder/email-icon-black.png',
                    'white' => IWETCL_IMAGES_URL . 'builder/email-icon-white.png'
                ],
                'PhoneIconSrc' => [
                    'black' => IWETCL_IMAGES_URL . 'builder/phone-icon-black.png',
                    'white' => IWETCL_IMAGES_URL . 'builder/phone-icon-white.png'
                ],
                'FacebookIconSrc' => [
                    'black' => IWETCL_IMAGES_URL . 'builder/facebook-black.png',
                    'white' => IWETCL_IMAGES_URL . 'builder/facebook-white.png',
                    'color' => IWETCL_IMAGES_URL . 'builder/facebook-color.png',
                    'white_border' => IWETCL_IMAGES_URL . 'builder/facebook-white-border.png',
                    'black_border' => IWETCL_IMAGES_URL . 'builder/facebook-black-border.png',
                    'color_border' => IWETCL_IMAGES_URL . 'builder/facebook-color-border.png',
                    'color_white' => IWETCL_IMAGES_URL . 'builder/facebook-color-white.png',
                    'black_white' => IWETCL_IMAGES_URL . 'builder/facebook-black-white.png',
                    'white_color' => IWETCL_IMAGES_URL . 'builder/facebook-white-color.png',
                ],
                'TwitterIconSrc' => [
                    'black' => IWETCL_IMAGES_URL . 'builder/twitter-black.png',
                    'white' => IWETCL_IMAGES_URL . 'builder/twitter-white.png',
                    'color' => IWETCL_IMAGES_URL . 'builder/twitter-color.png',
                    'white_border' => IWETCL_IMAGES_URL . 'builder/twitter-white-border.png',
                    'black_border' => IWETCL_IMAGES_URL . 'builder/twitter-black-border.png',
                    'color_border' => IWETCL_IMAGES_URL . 'builder/twitter-color-border.png',
                    'color_white' => IWETCL_IMAGES_URL . 'builder/twitter-color-white.png',
                    'black_white' => IWETCL_IMAGES_URL . 'builder/twitter-black-white.png',
                    'white_color' => IWETCL_IMAGES_URL . 'builder/twitter-white-color.png',
                ],
                'InstagramIconSrc' => [
                    'black' => IWETCL_IMAGES_URL . 'builder/instagram-black.png',
                    'white' => IWETCL_IMAGES_URL . 'builder/instagram-white.png',
                    'color' => IWETCL_IMAGES_URL . 'builder/instagram-color.png',
                    'white_border' => IWETCL_IMAGES_URL . 'builder/instagram-white-border.png',
                    'black_border' => IWETCL_IMAGES_URL . 'builder/instagram-black-border.png',
                    'color_border' => IWETCL_IMAGES_URL . 'builder/instagram-color-border.png',
                    'color_white' => IWETCL_IMAGES_URL . 'builder/instagram-color-white.png',
                    'black_white' => IWETCL_IMAGES_URL . 'builder/instagram-white-black.png',
                    'white_color' => IWETCL_IMAGES_URL . 'builder/instagram-white-color.png',
                ],
                'YoutubeIconSrc' => [
                    'black' => IWETCL_IMAGES_URL . 'builder/youtube-black.png',
                    'white' => IWETCL_IMAGES_URL . 'builder/youtube-white.png',
                    'color' => IWETCL_IMAGES_URL . 'builder/youtube-color.png',
                    'white_border' => IWETCL_IMAGES_URL . 'builder/youtube-white-border.png',
                    'black_border' => IWETCL_IMAGES_URL . 'builder/youtube-black-border.png',
                    'color_border' => IWETCL_IMAGES_URL . 'builder/youtube-color-border.png',
                    'color_white' => IWETCL_IMAGES_URL . 'builder/youtube-color-white.png',
                    'black_white' => IWETCL_IMAGES_URL . 'builder/youtube-black-white.png',
                    'white_color' => IWETCL_IMAGES_URL . 'builder/youtube-white-color.png',
                ],
                'WhatsappIconSrc' => [
                    'black' => IWETCL_IMAGES_URL . 'builder/whatsapp-black.png',
                    'white' => IWETCL_IMAGES_URL . 'builder/whatsapp-white.png',
                    'color' => IWETCL_IMAGES_URL . 'builder/whatsapp-color.png',
                    'white_border' => IWETCL_IMAGES_URL . 'builder/whatsapp-white-border.png',
                    'black_border' => IWETCL_IMAGES_URL . 'builder/whatsapp-black-border.png',
                    'color_border' => IWETCL_IMAGES_URL . 'builder/whatsapp-color-border.png',
                    'color_white' => IWETCL_IMAGES_URL . 'builder/whatsapp-color-white.png',
                    'black_white' => IWETCL_IMAGES_URL . 'builder/whatsapp-black-white.png',
                    'white_color' => IWETCL_IMAGES_URL . 'builder/whatsapp-white-color.png',
                ],
                'LinkedinIconSrc' => [
                    'black' => IWETCL_IMAGES_URL . 'builder/linkedin-black.png',
                    'white' => IWETCL_IMAGES_URL . 'builder/linkedin-white.png',
                    'color' => IWETCL_IMAGES_URL . 'builder/linkedin-color.png',
                    'white_border' => IWETCL_IMAGES_URL . 'builder/linkedin-white-border.png',
                    'black_border' => IWETCL_IMAGES_URL . 'builder/linkedin-black-border.png',
                    'color_border' => IWETCL_IMAGES_URL . 'builder/linkedin-color-border.png',
                    'color_white' => IWETCL_IMAGES_URL . 'builder/linkedin-color-white.png',
                    'black_white' => IWETCL_IMAGES_URL . 'builder/linkedin-black-white.png',
                    'white_color' => IWETCL_IMAGES_URL . 'builder/linkedin-white-color.png',
                ],
                'Products' => (!empty($products)) ? $products : [],
                'ProductCategories' => (!empty($product_categories)) ? $product_categories : [],
                'PostCategories' => (!empty($post_categories)) ? $post_categories : [],
                'PostIncluded' => (!empty($post_included)) ? $post_included : [],
                'PostExcluded' => (!empty($post_excluded)) ? $post_excluded : [],
                'AdminEmail' => get_option('admin_email', ''),
                'SiteUrl' => get_option('siteurl', ''),
                'PostData' => $this->post_data,
                'EmailTypes' => array_keys($this->email_type_repository->get_email_types()),
                'SampleTemplates' => $this->templates,
            ]);
            wp_enqueue_script('iwetc-component-defaults', IWETCL_JS_URL . 'builder/component-defaults.js', ['iwetc-builder'], IWETCL_VERSION);
            wp_enqueue_script('iwetc-properties', IWETCL_JS_URL . 'builder/properties.js', ['iwetc-builder'], IWETCL_VERSION);
            wp_enqueue_script('iwetc-components', IWETCL_JS_URL . 'builder/components.js', ['iwetc-builder'], IWETCL_VERSION);
            wp_enqueue_script('iwetc-functions', IWETCL_JS_URL . 'builder/functions.js', ['iwetc-components', 'wp-color-picker', 'iwetc-select2'], IWETCL_VERSION);
            wp_enqueue_script('iwetc-modal', IWETCL_JS_URL . 'common/modal.js', ['jquery'], IWETCL_VERSION);
            wp_enqueue_script('iwetc-builder-events', IWETCL_JS_URL . 'builder/events.js', ['iwetc-components'], IWETCL_VERSION);
            wp_enqueue_script('iwetc-main', IWETCL_JS_URL . 'common/main.js', ['jquery'], IWETCL_VERSION);
            wp_localize_script('iwetc-main', 'IWETCL_DATA', [
                'ajax_url' => admin_url('admin-ajax.php'),
                'wp_nonce' => $this->nonce,
            ]);
        }
    }

    private function get_builder_translations()
    {
        return [
            'layout' => __('Layout', 'ithemeland-woocommerce-email-template-customizer-lite'),
            'general' => __('General', 'ithemeland-woocommerce-email-template-customizer-lite'),
            'content' => __('Content', 'ithemeland-woocommerce-email-template-customizer-lite'),
            'woocommerce' => __('Woocommerce', 'ithemeland-woocommerce-email-template-customizer-lite'),
            'one_column' => __('1 Column', 'ithemeland-woocommerce-email-template-customizer-lite'),
            'two_columns' => __('2 Columns', 'ithemeland-woocommerce-email-template-customizer-lite'),
            'three_columns' => __('3 Columns', 'ithemeland-woocommerce-email-template-customizer-lite'),
            'four_columns' => __('4 Columns', 'ithemeland-woocommerce-email-template-customizer-lite'),
            'text' => __('Text', 'ithemeland-woocommerce-email-template-customizer-lite'),
            'html' => __('Html', 'ithemeland-woocommerce-email-template-customizer-lite'),
            'width' => __('Width', 'ithemeland-woocommerce-email-template-customizer-lite'),
            'image' => __('Image', 'ithemeland-woocommerce-email-template-customizer-lite'),
            'button' => __('Button', 'ithemeland-woocommerce-email-template-customizer-lite'),
            'order_details' => __('Order details', 'ithemeland-woocommerce-email-template-customizer-lite'),
            'order_subtotal' => __('Order subtotal', 'ithemeland-woocommerce-email-template-customizer-lite'),
            'order_total' => __('Order total', 'ithemeland-woocommerce-email-template-customizer-lite'),
            'shipping_method' => __('Shipping method', 'ithemeland-woocommerce-email-template-customizer-lite'),
            'payment_method' => __('Payment method', 'ithemeland-woocommerce-email-template-customizer-lite'),
            'customer_note' => __('Customer note', 'ithemeland-woocommerce-email-template-customizer-lite'),
            'billing_address' => __('Billing address', 'ithemeland-woocommerce-email-template-customizer-lite'),
            'shipping_address' => __('Shipping address', 'ithemeland-woocommerce-email-template-customizer-lite'),
            'products' => __('Products', 'ithemeland-woocommerce-email-template-customizer-lite'),
            'coupon' => __('Coupon', 'ithemeland-woocommerce-email-template-customizer-lite'),
            'coupon_type' => __('Coupon type', 'ithemeland-woocommerce-email-template-customizer-lite'),
            'post' => __('Post', 'ithemeland-woocommerce-email-template-customizer-lite'),
            'contact' => __('Contact', 'ithemeland-woocommerce-email-template-customizer-lite'),
            'menu_bar' => __('Menu Bar', 'ithemeland-woocommerce-email-template-customizer-lite'),
            'socials' => __('Socials', 'ithemeland-woocommerce-email-template-customizer-lite'),
            'divider' => __('Divider', 'ithemeland-woocommerce-email-template-customizer-lite'),
            'spacer' => __('Spacer', 'ithemeland-woocommerce-email-template-customizer-lite'),
            'header' => __('Header', 'ithemeland-woocommerce-email-template-customizer-lite'),
            'wc_hook' => __('WC Hook', 'ithemeland-woocommerce-email-template-customizer-lite'),
            'style' => __('Style', 'ithemeland-woocommerce-email-template-customizer-lite'),
            'custom_class' => __('Custom class', 'ithemeland-woocommerce-email-template-customizer-lite'),
            'text_editor' => __('Text editor', 'ithemeland-woocommerce-email-template-customizer-lite'),
            'html_editor' => __('Html editor', 'ithemeland-woocommerce-email-template-customizer-lite'),
            'column' => __('Column', 'ithemeland-woocommerce-email-template-customizer-lite'),
            'columns' => __('Columns', 'ithemeland-woocommerce-email-template-customizer-lite'),
            'margin' => __('Margin', 'ithemeland-woocommerce-email-template-customizer-lite'),
            'padding' => __('Padding', 'ithemeland-woocommerce-email-template-customizer-lite'),
            'background_color' => __('Background color', 'ithemeland-woocommerce-email-template-customizer-lite'),
            'background_image' => __('Background image', 'ithemeland-woocommerce-email-template-customizer-lite'),
            'background' => __('Background', 'ithemeland-woocommerce-email-template-customizer-lite'),
            'text_color' => __('Text color', 'ithemeland-woocommerce-email-template-customizer-lite'),
            'height' => __('Height', 'ithemeland-woocommerce-email-template-customizer-lite'),
            'line_height' => __('Line height', 'ithemeland-woocommerce-email-template-customizer-lite'),
            'border' => __('Border', 'ithemeland-woocommerce-email-template-customizer-lite'),
            'edit' => __('Edit', 'ithemeland-woocommerce-email-template-customizer-lite'),
            'move' => __('Move', 'ithemeland-woocommerce-email-template-customizer-lite'),
            'duplicate' => __('Duplicate', 'ithemeland-woocommerce-email-template-customizer-lite'),
            'copy' => __('Copy', 'ithemeland-woocommerce-email-template-customizer-lite'),
            'paste' => __('Paste', 'ithemeland-woocommerce-email-template-customizer-lite'),
            'delete' => __('Delete', 'ithemeland-woocommerce-email-template-customizer-lite'),
            'image' => __('Image', 'ithemeland-woocommerce-email-template-customizer-lite'),
            'alignment' => __('Alignment', 'ithemeland-woocommerce-email-template-customizer-lite'),
            'size' => __('Size', 'ithemeland-woocommerce-email-template-customizer-lite'),
            'font' => __('Font', 'ithemeland-woocommerce-email-template-customizer-lite'),
            'text' => __('Text', 'ithemeland-woocommerce-email-template-customizer-lite'),
            'link' => __('Link', 'ithemeland-woocommerce-email-template-customizer-lite'),
            'template' => __('Template', 'ithemeland-woocommerce-email-template-customizer-lite'),
            'translate' => __('Translate', 'ithemeland-woocommerce-email-template-customizer-lite'),
            'show_sku' => __('Show SKU', 'ithemeland-woocommerce-email-template-customizer-lite'),
            'items_distance' => __('Items distance', 'ithemeland-woocommerce-email-template-customizer-lite'),
            'distance' => __('distance', 'ithemeland-woocommerce-email-template-customizer-lite'),
            'images_size' => __('Images size', 'ithemeland-woocommerce-email-template-customizer-lite'),
            'product_name' => __('Product name', 'ithemeland-woocommerce-email-template-customizer-lite'),
            'quantity' => __('Quantity', 'ithemeland-woocommerce-email-template-customizer-lite'),
            'price' => __('Price', 'ithemeland-woocommerce-email-template-customizer-lite'),
            'product_type' => __('Product type', 'ithemeland-woocommerce-email-template-customizer-lite'),
            'product_image' => __('Product image', 'ithemeland-woocommerce-email-template-customizer-lite'),
            'post_image' => __('Post image', 'ithemeland-woocommerce-email-template-customizer-lite'),
            'post_title' => __('Post title', 'ithemeland-woocommerce-email-template-customizer-lite'),
            'post_content' => __('Post content', 'ithemeland-woocommerce-email-template-customizer-lite'),
            'post_type' => __('Post type', 'ithemeland-woocommerce-email-template-customizer-lite'),
            'contact_home' => __('Home url', 'ithemeland-woocommerce-email-template-customizer-lite'),
            'icon_size' => __('Icon size', 'ithemeland-woocommerce-email-template-customizer-lite'),
            'contact_email' => __('Email', 'ithemeland-woocommerce-email-template-customizer-lite'),
            'contact_phone' => __('Phone', 'ithemeland-woocommerce-email-template-customizer-lite'),
            'contact_items' => __('Items', 'ithemeland-woocommerce-email-template-customizer-lite'),
            'direction' => __('Direction', 'ithemeland-woocommerce-email-template-customizer-lite'),
            'select_hook' => __('Select Hook', 'ithemeland-woocommerce-email-template-customizer-lite'),
            'menu_bar_items' => __('Items', 'ithemeland-woocommerce-email-template-customizer-lite'),
            'menu_bar_divider' => __('Divider', 'ithemeland-woocommerce-email-template-customizer-lite'),
            'social_items' => __('Items', 'ithemeland-woocommerce-email-template-customizer-lite'),
            'product_add_to_cart_in_url' => __('Add to cart in url', 'ithemeland-woocommerce-email-template-customizer-lite'),
            'border_radius' => __('Border radius', 'ithemeland-woocommerce-email-template-customizer-lite'),
            'style' => __('Style', 'ithemeland-woocommerce-email-template-customizer-lite'),
        ];
    }

    private function get_post_data()
    {
        if (empty($_GET['post'])) {
            return '';
        }

        $postmeta = get_post_meta(intval($_GET['post']));
        return [
            'email_type' => (!empty($postmeta['iwetc_email_type'][0])) ? $postmeta['iwetc_email_type'][0] : '',
            'direction' => (!empty($postmeta['iwetc_direction'][0])) ? $postmeta['iwetc_direction'][0] : '',
            'custom_css' => (!empty($postmeta['iwetc_custom_css'][0])) ? $postmeta['iwetc_custom_css'][0] : '',
            'template_is_enable' => (!empty($postmeta['iwetc_template_enable'][0])) ? $postmeta['iwetc_template_enable'][0] : '',
            'email_structure' => (!empty($postmeta['iwetc_email_structure'][0])) ? $postmeta['iwetc_email_structure'][0] : '',
        ];
    }
}
