<?php

namespace iwetcl\classes\services\render;

use iwetcl\classes\helpers\Sanitizer;
use iwetcl\classes\helpers\Shortcode_Helper;
use iwetcl\classes\repositories\Order;
use iwetcl\classes\repositories\User;

class Email_Render
{
    private $email_structure;
    private $render_methods;
    private $element_render;
    private $order_object;
    private $user_object;
    private $render_type;
    private $custom_css;

    public function __construct($data)
    {
        $this->user_object = null;
        $this->email_structure = (!is_array($data['email_structure'])) ? json_decode($data['email_structure'], true) : $data['email_structure'];
        $this->custom_css = (!empty($data['custom_css'])) ? $data['custom_css'] : '';
        $this->direction = (!empty($data['direction'])) ? $data['direction'] : 'ltr';
        $this->render_methods = $this->get_render_methods();
        $this->element_render = Element_Render::get_instance();
        $this->render_type = sanitize_text_field($data['render_type']);

        if ($data['object_type'] == 'order' && $data['object'] instanceof \WC_Order) {
            $this->order_object = $data['object'];
            $this->user_object = $this->order_object->get_user();
        }

        if ($data['object_type'] == 'order_id') {
            $order_repository = Order::get_instance();
            $this->order_object = $order_repository->get_order(intval($data['object']));
            $this->user_object = $this->order_object->get_user();
        }

        if ($data['object_type'] == 'user' && $data['object'] instanceof \WP_User) {
            $this->user_object = $data['object'];
        }

        if ($data['object_type'] == 'user_id') {
            $user_repository = User::get_instance();
            $this->user_object = $user_repository->get_user(intval($data['object']));
        }
    }

    public function render()
    {
        if (empty($this->email_structure)) {
            return false;
        }

        $this->shortcode = Shortcode_Helper::get_instance();
        $this->shortcode->set_data([
            'user_object' => $this->user_object,
            'order_object' => (!empty($this->order_object)) ? $this->order_object : null,
            'password_reset_key' => true
        ]);

        $output = $this->get_header();

        if (!empty($this->email_structure['rows']) && is_array($this->email_structure['rows'])) {
            foreach ($this->email_structure['rows'] as $row) {
                $row_custom_class = (!empty($row['layout']['layout_content_attributes']['data-custom-class'])) ? esc_attr($row['layout']['layout_content_attributes']['data-custom-class']) : '';
                $distance = (!empty($row['layout']['distance'])) ? intval($row['layout']['distance']) : 0;
                $output .= '<div class="iwetc-element-wrapper ' . Sanitizer::attribute($row_custom_class) . '" style="box-sizing: border-box; -moz-box-sizing: border-box; -webkit-box-sizing: border-box; border-style: solid;' . Sanitizer::attribute($row['layout']['layout_content_attributes']['style']) . '">';
                if (!empty($row['columns']) && is_array($row['columns'])) {
                    $i = 0;
                    foreach ($row['columns'] as $column) {
                        $width = (!empty($column['attributes']['data-width'])) ? intval($column['attributes']['data-width']) . '%' : 'auto';
                        $column_custom_class = (!empty($column['attributes']['data-custom-class'])) ? esc_attr($column['attributes']['data-custom-class']) : '';
                        $margin_left = 0;
                        if ($distance != 0 && $width != 'auto') {
                            $diff = $distance - ($distance / intval(count($row['columns'])));
                            $width = 'calc(' . $width . ' - ' . $diff . 'px)';
                            $margin_left = ($i == 0) ? 0 : $distance . 'px';
                        }

                        $output .= '<div class="iwetc-column ' . Sanitizer::attribute($column_custom_class) . '" style="width: ' . $width . '; min-height: 27px; margin-left: ' . $margin_left . '; ' . Sanitizer::attribute($column['attributes']['style']) . '">';
                        if (!empty($column['contents']) && is_array($column['contents'])) {
                            foreach ($column['contents'] as $content) {
                                if (!empty($content['name'])) {
                                    $content['render_type'] = $this->render_type;
                                    $output .= $this->content_render($content);
                                }
                            }
                        }
                        $output .= '</div>';
                        $i++;
                    }
                }
                $output .= '</div>';
            }
        }

        $output .= $this->get_footer();

        return $output;
    }

    private function get_render_methods()
    {
        return [
            'text' => 'custom_render',
            'html' => 'custom_render',
            'image' => 'basic_render',
            'button' => 'basic_render',
            'order_details' => 'custom_render',
            'order_subtotal' => 'custom_render',
            'order_total' => 'custom_render',
            'shipping_method' => 'custom_render',
            'payment_method' => 'custom_render',
            'customer_note' => 'custom_render',
            'billing_address' => 'custom_render',
            'shipping_address' => 'custom_render',
            'coupon' => 'basic_render',
            'coupon' => 'custom_render',
            'products' => 'custom_render',
            'post' => 'custom_render',
            'contact' => 'basic_render',
            'menu_bar' => 'basic_render',
            'socials' => 'basic_render',
            'divider' => 'basic_render',
            'spacer' => 'basic_render',
            'wc_hook' => 'custom_render',
        ];
    }

    private function content_render($content)
    {
        if (empty($content['name'])) {
            return '';
        }

        $render_method = (!empty($this->render_methods[$content['name']]) && method_exists($this, $this->render_methods[$content['name']])) ? sanitize_text_field($this->render_methods[$content['name']]) : null;
        if (empty($render_method)) {
            return '';
        }

        $custom_class_in_html = (!empty($content['custom_class_in_html'])) ? esc_attr($content['custom_class_in_html']) : '';
        $content['html'] = str_replace('iwetc-cc-285158736', $custom_class_in_html, $content['html']);

        $rendered = $this->{$render_method}($content);
        $style = (!empty($content['element_content_attributes']['style'])) ? $content['element_content_attributes']['style'] : "";
        $custom_class = (!empty($content['element_content_attributes']['data-custom-class'])) ? Sanitizer::attribute($content['element_content_attributes']['data-custom-class']) : '';
        $output = '<div class="' . Sanitizer::attribute($custom_class) . '" style="width: 100%; float: left;' . Sanitizer::attribute($style) . '">';
        $output .= $this->shortcode->replace($rendered);
        $output .= '</div>';

        return $output;
    }

    private function basic_render($content)
    {
        return sprintf('%s', wp_unslash($content['html']));
    }

    private function custom_render($content)
    {
        $method_name = sanitize_text_field('get_' . $content['name']);
        if (!method_exists($this->element_render, $method_name)) {
            return '';
        }

        $output = $this->element_render->{$method_name}([
            'render_type' => $this->render_type,
            'order' => (!empty($this->order_object)) ? $this->order_object : '',
            'translate' => (!empty($content['translate'])) ? $content['translate'] : '',
            'attributes' => (!empty($content['attributes'])) ? $content['attributes'] : '',
            'element_content_attributes' => (!empty($content['element_content_attributes'])) ? $content['element_content_attributes'] : '',
            'children_attributes' => (!empty($content['children_attributes'])) ? $content['children_attributes'] : '',
            'html' => (!empty($content['html'])) ? $content['html'] : '',
        ]);

        return sprintf('%s', wp_unslash($output));
    }

    private function get_header()
    {
        $container_style = (!empty($this->email_structure['container']['style'])) ? Sanitizer::attribute($this->email_structure['container']['style']) : '';
        $wrapper_style = (!empty($this->email_structure['wrapper']['style'])) ? Sanitizer::attribute($this->email_structure['wrapper']['style']) : '';

        $output = '<!DOCTYPE html><html xmlns="http://www.w3.org/1999/xhtml"><head>';
        $output .= '<meta http-equiv="Content-Type" content="text/html; charset=UTF-8"><meta name="viewport" content="width=device-width, initial-scale=1">';
        $output .= $this->get_default_style();
        $output .= '</head><body>';
        $output .= '<div style="direction: ' . $this->direction . '; width: 100%; ' . $container_style . '"><div class="iwetc-email-wrapper" style="margin: auto; ' . $wrapper_style . '">';
        $output .= '<div style="width: 100%; display: inline-block; box-sizing: border-box; -moz-box-sizing: border-box; -webkit-box-sizing: border-box;">';

        return $output;
    }

    private function get_footer()
    {
        return '</div></div></div></body></html>';
    }

    private function get_default_style()
    {
        $output = '<style type="text/css">
        * {border: 0; word-break: break-all; font-family: -apple-system,BlinkMacSystemFont,"Segoe UI",Roboto,Oxygen-Sans,Ubuntu,Cantarell,"Helvetica Neue",sans-serif} 
        body {
            margin: 0;
            padding: 0;
            -webkit-text-size-adjust: 100%;
            -ms-text-size-adjust: 100%;
        }
        table, td {
            mso-table-lspace: 0pt;
            mso-table-rspace: 0pt;
        }
        h1, h2, h3, h4, h5, h6 {
            line-height: inherit !important;
            padding: 0;
            margin: 0;
        }
        h1{font-size: 23px}
        p {padding: 0; margin: 0; line-height: inherit; width: 100%; float: left;} 
        p img {max-width: 100%; height: auto}
        a {text-decoration: none;} 
        ul{padding: 0; margin: 0; list-style: none;} 
        ul li{margin-left: 0;}
        .iwetc-mobile .iwetc-table-responsive{width: 100% !important;}
        .iwetc-mobile .iwetc-column{width: 100% !important; margin-bottom: 10px;}
        .iwetc-column {min-height: 27px;}
        .iwetc-column iframe{max-width: 100% !important;}
        .aligncenter {display: block; margin: auto; text-align: center;}
        .alignleft {text-align: left;}
        .alignright {text-align: right;}
        .iwetc-element-wrapper{padding: 15px 25px;}

        ul.iwetc-socials-style2 li {
            padding: 5px;
            box-sizing: border-box;
            -moz-box-sizing: border-box;
            -webkit-box-sizing: border-box;
            margin: 0 2px;
        }
        
        ul.iwetc-socials-style2 li:first-child {
            margin-left: 0;
        }
        
        ul.iwetc-socials-style2 li:last-child {
            margin-right: 0;
        }
        
        ul.iwetc-socials-style2 li.iwetc-socials-facebook {
            background-color: #4267B2;
        }
        
        ul.iwetc-socials-style2 li.iwetc-socials-twitter {
            background-color: #00acee;
        }
        
        ul.iwetc-socials-style2 li.iwetc-socials-youtube {
            background-color: #ff0000;
        }
        
        ul.iwetc-socials-style2 li.iwetc-socials-instagram {
            background-color: #8a3ab9;
        }
        
        ul.iwetc-socials-style2 li.iwetc-socials-linkedin {
            background-color: #0077b5;
        }
        
        ul.iwetc-socials-style2 li.iwetc-socials-whatsapp {
            background-color: #128c7e;
        }

        @media only screen and (max-width: 480px) {
            .iwetc-column{width: 100% !important; margin-bottom: 10px;}
            .iwetc-email-wrapper{width: 360px !important;}
            .iwetc-element-wrapper{padding: 8px !important;}
            .iwetc-table-responsive{width: 100% !important;}
        }
        
        ';
        if (!empty($this->custom_css)) {
            $output .= $this->custom_css;
        }
        $output .= '</style>';
        return $output;
    }
}
